library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity waitgen is
port
	(
	datain		: in std_logic_vector(7 downto 0);
--	sysclk		: in std_logic;
	cpuclk		: in std_logic;
--	reset		: in std_logic;
	dma			: in std_logic;
	wr			: in std_logic;
	iorq		: in std_logic;
	speed		: in std_logic;
	rd			: in std_logic;
	Addr		: in std_logic_vector(15 downto 0);
	disp		: in std_logic;
	mreq		: in std_logic;
	m1			: in std_logic;
	dataout		: out std_logic_vector(7 downto 0);
	Csel		: out std_logic_vector(3 downto 0);
	selects		: out std_logic_vector(3 downto 0);
	waitout		: out std_logic;
	resetout	: out std_logic
	);
end;

architecture rtl of waitgen is


signal i_csel	     :std_logic_vector(3 downto 0);
signal cena, cenad	:std_logic;
signal waitcnt		:std_logic_vector(3 downto 0);
signal cpu			:std_logic;
signal IOena		:std_logic;
signal cpuwait		:std_logic_vector(1 downto 0);
signal ide2ena		:std_logic;
signal ide1ena		:std_logic;
signal fdcena		:std_logic;
signal i_selects	:std_logic_vector(2 downto 0);
signal tover		:std_logic;
signal reset		:std_logic;
signal dmad			:std_logic;
signal iowait		:std_logic_vector(3 downto 0);


begin
	cena <= IOena or ide1ena or ide2ena or fdcena;
	IOena <= '1' when (iorq='0' and ((rd and wr)='0')and   addr(15 downto 5)="11111100000") else '0';
	ide1ena <= '1' when (iorq='0' and ((rd and wr)='0')and addr(15 downto 4)="111111010100") else '0';
	ide2ena <= '1' when (iorq='0' and ((rd and wr)='0')and addr(15 downto 4)="111111010000") else '0';
	fdcena <= '1' when (iorq='0' and ((rd and wr)='0')and  addr(15 downto 1)="111110110111111") else '0';
				i_selects <= 	"011" when fdcena='1' else
								"101" when ide2ena='1' else
								"100" when ide1ena='1' else
								"110" when IOena='1' else "111";	
	
--	waitout <= ((NOT cena AND (cpu OR (mreq AND m1))) OR tover) AND dma; --dma=0 ... der 1K30 ist noch nicht so weit
--	waitout <= ((NOT cena AND (cpu)) OR tover) AND dma; --dma=0 ... der 1K30 ist noch nicht so weit
--	waitout <= ((NOT cena AND ((iowait(1) OR mreq) AND (iowait(3) OR iorq OR NOT m1) AND (iowait(2) OR m1 OR NOT mreq))) OR tover) AND dma; --dma=0 ... der 1K30 ist noch nicht so weit
	waitout <= ((NOT cena AND (iowait(1) OR (mreq AND  iorq ))) OR tover) AND dma; --dma=0 ... der 1K30 ist noch nicht so weit
	resetout <= reset;
	
	process (cpuclk) begin
			if (cpuclk'event and cpuclk='1') then
--				iowait <= cpu;
				if cpuwait/=3 and speed='1'  then
--					cpu <= '0';
					iowait <= iowait(2 downto 0)&'0';
				else
--					cpu <= '1';
					iowait <= iowait(2 downto 0)&'1';
				end if;	
				cpuwait <= cpuwait+1;
				dmad <= dma;
				if dma='1' AND dmad='0' then
					reset <= '1';  --beim ersten dma lh wird reset aufgehoben
				end if;	
			end if;
	end process;
	
	process (cpuclk, reset) begin
		if reset = '0' then
			tover <= '0';
			waitcnt <= (others=>'0');
			selects <= (others=>'1');
		else
			if (cpuclk'event and cpuclk='1') then
				cenad <= cena;	--Flankendetektor
				if iorq='1' then
					tover <= '0';
				end if;	
				
				if (cena='1' and cenad='0') then --rising edge
					waitcnt <= "1111";--(others=>'1'); 07 geht nicht, 0c geht
					selects <=(rd&i_selects);
					if IOena='1' then
						csel <= i_csel;
					else
						csel <= "1111";	
					end if;
				end if;
				if (waitcnt/=0) then
					waitcnt <= waitcnt-1;	
				end if;
				if (waitcnt=2)then	
					dataout <= (others => 'Z');
				end if;
				if (waitcnt=1)then	--evtl 0
--					selects <="1111";
					csel <= "1111";
					tover <= '1';		--langes wait
--					dataout <= (others => 'Z');
				end if;
				if (waitcnt=3)then
					selects <="1111";
					if rd='0' then
						dataout <= datain;
						tover <= '1';
						csel <= "1111";
					end if;							
				end if;
--				if (waitcnt=5)then
--					if rd='0' then
--						dataout <= datain;
--					end if;							
--				end if;
			end if;
		end if;
	end process;
	
	
	process (cpuclk, reset) begin
		if reset = '0' then
			i_csel <= "1111";
		else
			if (cpuclk'event and cpuclk='1') then
				if (iorq='0' and ((rd and wr)='0')and addr(15 downto 4)=x"FC2") then --Csel setzen
					i_csel <= addr(3 downto 0);
				end if;
			end if;	
		end if;	
	end process;	
end;
